@@ -56,9 +56,10 @@ MSWORD_FRIENDLY = 11
 PLAIN_COLUMNS = 12
 MARKDOWN = 13
 ORGMODE = 14
+DOUBLE_BORDER = 15
 RANDOM = 20
 
-_re = re.compile(r"\033\[[0-9;]*m")
+_re = re.compile(r"\033\[[0-9;]*m|\033\(B")
 
 
 def _get_size(text):
@@ -91,6 +92,7 @@ class PrettyTable:
             Allowed values: FRAME, ALL, NONE
         int_format - controls formatting of integer data
         float_format - controls formatting of floating point data
+        custom_format - controls formatting of any column using callable
         min_table_width - minimum desired table width, in characters
         max_table_width - maximum desired table width, in characters
         min_width - minimum desired field width, in characters
@@ -101,7 +103,21 @@ class PrettyTable:
         right_padding_width - number of spaces on right hand side of column data
         vertical_char - single character string used to draw vertical lines
         horizontal_char - single character string used to draw horizontal lines
+        horizontal_align_char - single character string used to indicate alignment
         junction_char - single character string used to draw line junctions
+        top_junction_char - single character string used to draw top line junctions
+        bottom_junction_char -
+            single character string used to draw bottom line junctions
+        right_junction_char - single character string used to draw right line junctions
+        left_junction_char - single character string used to draw left line junctions
+        top_right_junction_char -
+            single character string used to draw top-right line junctions
+        top_left_junction_char -
+            single character string used to draw top-left line junctions
+        bottom_right_junction_char -
+            single character string used to draw bottom-right line junctions
+        bottom_left_junction_char -
+            single character string used to draw bottom-left line junctions
         sortby - name of field to sort rows by
         sort_key - sorting key function, applied to data points before sorting
         align - default align for each column (None, "l", "c" or "r")
@@ -120,25 +136,58 @@ class PrettyTable:
         self.min_width = {}
         self.int_format = {}
         self.float_format = {}
+        self.custom_format = {}
+
         if field_names:
             self.field_names = field_names
         else:
             self._widths = []
 
         # Options
-        self._options = (
-            "title start end fields header border sortby reversesort "
-            "sort_key attributes format hrules vrules".split()
-        )
-        self._options.extend(
-            "int_format float_format min_table_width max_table_width padding_width "
-            "left_padding_width right_padding_width".split()
-        )
-        self._options.extend(
-            "vertical_char horizontal_char junction_char header_style valign xhtml "
-            "print_empty oldsortslice".split()
-        )
-        self._options.extend("align valign max_width min_width".split())
+        self._options = [
+            "title",
+            "start",
+            "end",
+            "fields",
+            "header",
+            "border",
+            "sortby",
+            "reversesort",
+            "sort_key",
+            "attributes",
+            "format",
+            "hrules",
+            "vrules",
+            "int_format",
+            "float_format",
+            "custom_format",
+            "min_table_width",
+            "max_table_width",
+            "padding_width",
+            "left_padding_width",
+            "right_padding_width",
+            "vertical_char",
+            "horizontal_char",
+            "horizontal_align_char",
+            "junction_char",
+            "header_style",
+            "valign",
+            "xhtml",
+            "print_empty",
+            "oldsortslice",
+            "top_junction_char",
+            "bottom_junction_char",
+            "right_junction_char",
+            "left_junction_char",
+            "top_right_junction_char",
+            "top_left_junction_char",
+            "bottom_right_junction_char",
+            "bottom_left_junction_char",
+            "align",
+            "valign",
+            "max_width",
+            "min_width",
+        ]
         for option in self._options:
             if option in kwargs:
                 self._validate_option(option, kwargs[option])
@@ -176,6 +225,7 @@ class PrettyTable:
         self.min_width = kwargs["min_width"] or {}
         self.int_format = kwargs["int_format"] or {}
         self.float_format = kwargs["float_format"] or {}
+        self.custom_format = kwargs["custom_format"] or {}
 
         self._min_table_width = kwargs["min_table_width"] or None
         self._max_table_width = kwargs["max_table_width"] or None
@@ -188,7 +238,16 @@ class PrettyTable:
 
         self._vertical_char = kwargs["vertical_char"] or "|"
         self._horizontal_char = kwargs["horizontal_char"] or "-"
+        self._horizontal_align_char = kwargs["horizontal_align_char"]
         self._junction_char = kwargs["junction_char"] or "+"
+        self._top_junction_char = kwargs["top_junction_char"]
+        self._bottom_junction_char = kwargs["bottom_junction_char"]
+        self._right_junction_char = kwargs["right_junction_char"]
+        self._left_junction_char = kwargs["left_junction_char"]
+        self._top_right_junction_char = kwargs["top_right_junction_char"]
+        self._top_left_junction_char = kwargs["top_left_junction_char"]
+        self._bottom_right_junction_char = kwargs["bottom_right_junction_char"]
+        self._bottom_left_junction_char = kwargs["bottom_left_junction_char"]
 
         if kwargs["print_empty"] in (True, False):
             self._print_empty = kwargs["print_empty"]
@@ -256,6 +315,16 @@ class PrettyTable:
     def __str__(self):
         return self.get_string()
 
+    def __repr__(self):
+        return self.get_string()
+
+    def _repr_html_(self):
+        """
+        Returns get_html_string value by default
+        as the repr call in Jupyter notebook environment
+        """
+        return self.get_html_string()
+
     ##############################
     # ATTRIBUTE VALIDATORS       #
     ##############################
@@ -310,7 +379,23 @@ class PrettyTable:
             self._validate_int_format(option, val)
         elif option == "float_format":
             self._validate_float_format(option, val)
-        elif option in ("vertical_char", "horizontal_char", "junction_char"):
+        elif option == "custom_format":
+            for k, formatter in val.items():
+                self._validate_function(f"{option}.{k}", formatter)
+        elif option in (
+            "vertical_char",
+            "horizontal_char",
+            "horizontal_align_char",
+            "junction_char",
+            "top_junction_char",
+            "bottom_junction_char",
+            "right_junction_char",
+            "left_junction_char",
+            "top_right_junction_char",
+            "top_left_junction_char",
+            "bottom_right_junction_char",
+            "bottom_left_junction_char",
+        ):
             self._validate_single_char(option, val)
         elif option == "attributes":
             self._validate_attributes(option, val)
@@ -337,39 +422,39 @@ class PrettyTable:
         try:
             assert len(val) == len(set(val))
         except AssertionError:
-            raise Exception("Field names must be unique!")
+            raise Exception("Field names must be unique")
 
     def _validate_header_style(self, val):
         try:
             assert val in ("cap", "title", "upper", "lower", None)
         except AssertionError:
             raise Exception(
-                "Invalid header style, use cap, title, upper, lower or None!"
+                "Invalid header style, use cap, title, upper, lower or None"
             )
 
     def _validate_align(self, val):
         try:
             assert val in ["l", "c", "r"]
         except AssertionError:
-            raise Exception(f"Alignment {val} is invalid, use l, c or r!")
+            raise Exception(f"Alignment {val} is invalid, use l, c or r")
 
     def _validate_valign(self, val):
         try:
             assert val in ["t", "m", "b", None]
         except AssertionError:
-            raise Exception(f"Alignment {val} is invalid, use t, m, b or None!")
+            raise Exception(f"Alignment {val} is invalid, use t, m, b or None")
 
     def _validate_nonnegative_int(self, name, val):
         try:
             assert int(val) >= 0
         except AssertionError:
-            raise Exception(f"Invalid value for {name}: {val}!")
+            raise Exception(f"Invalid value for {name}: {val}")
 
     def _validate_true_or_false(self, name, val):
         try:
             assert val in (True, False)
         except AssertionError:
-            raise Exception(f"Invalid value for {name}! Must be True or False.")
+            raise Exception(f"Invalid value for {name}. Must be True or False.")
 
     def _validate_int_format(self, name, val):
         if val == "":
@@ -379,7 +464,7 @@ class PrettyTable:
             assert val.isdigit()
         except AssertionError:
             raise Exception(
-                f"Invalid value for {name}! Must be an integer format string."
+                f"Invalid value for {name}. Must be an integer format string."
             )
 
     def _validate_float_format(self, name, val):
@@ -397,56 +482,69 @@ class PrettyTable:
                 or (bits[1][-1] == "f" and bits[1].rstrip("f").isdigit())
             )
         except AssertionError:
-            raise Exception(f"Invalid value for {name}! Must be a float format string.")
+            raise Exception(f"Invalid value for {name}. Must be a float format string.")
 
     def _validate_function(self, name, val):
         try:
             assert hasattr(val, "__call__")
         except AssertionError:
-            raise Exception(f"Invalid value for {name}! Must be a function.")
+            raise Exception(f"Invalid value for {name}. Must be a function.")
 
     def _validate_hrules(self, name, val):
         try:
             assert val in (ALL, FRAME, HEADER, NONE)
         except AssertionError:
             raise Exception(
-                f"Invalid value for {name}! Must be ALL, FRAME, HEADER or NONE."
+                f"Invalid value for {name}. Must be ALL, FRAME, HEADER or NONE."
             )
 
     def _validate_vrules(self, name, val):
         try:
             assert val in (ALL, FRAME, NONE)
         except AssertionError:
-            raise Exception(f"Invalid value for {name}! Must be ALL, FRAME, or NONE.")
+            raise Exception(f"Invalid value for {name}. Must be ALL, FRAME, or NONE.")
 
     def _validate_field_name(self, name, val):
         try:
             assert (val in self._field_names) or (val is None)
         except AssertionError:
-            raise Exception(f"Invalid field name: {val}!")
+            raise Exception(f"Invalid field name: {val}")
 
     def _validate_all_field_names(self, name, val):
         try:
             for x in val:
                 self._validate_field_name(name, x)
         except AssertionError:
-            raise Exception("fields must be a sequence of field names!")
+            raise Exception("Fields must be a sequence of field names")
 
     def _validate_single_char(self, name, val):
         try:
             assert _str_block_width(val) == 1
         except AssertionError:
-            raise Exception(f"Invalid value for {name}! Must be a string of length 1.")
+            raise Exception(f"Invalid value for {name}. Must be a string of length 1.")
 
     def _validate_attributes(self, name, val):
         try:
             assert isinstance(val, dict)
         except AssertionError:
-            raise Exception("attributes must be a dictionary of name/value pairs!")
+            raise Exception("Attributes must be a dictionary of name/value pairs")
 
     ##############################
     # ATTRIBUTE MANAGEMENT       #
     ##############################
+    @property
+    def rows(self):
+        return self._rows[:]
+
+    @property
+    def xhtml(self):
+        """Print <br/> tags if True, <br> tags if False"""
+        return self._xhtml
+
+    @xhtml.setter
+    def xhtml(self, val):
+        self._validate_option("xhtml", val)
+        self._xhtml = val
 
     @property
     def field_names(self):
@@ -774,6 +872,31 @@ class PrettyTable:
             for field in self._field_names:
                 self._float_format[field] = val
 
+    @property
+    def custom_format(self):
+        """Controls formatting of any column using callable
+        Arguments:
+
+        custom_format - Dictionary of field_name and callable"""
+        return self._custom_format
+
+    @custom_format.setter
+    def custom_format(self, val):
+        if val is None:
+            self._custom_format = {}
+        elif isinstance(val, dict):
+            for k, v in val.items():
+                self._validate_function(f"custom_value.{k}", v)
+            self._custom_format = val
+        elif hasattr(val, "__call__"):
+            self._validate_function("custom_value", val)
+            for field in self._field_names:
+                self._custom_format[field] = val
+        else:
+            raise Exception(
+                "The custom_format property need to be a dictionary or callable"
+            )
+
     @property
     def padding_width(self):
         """The number of empty spaces between a column's edge and its content
@@ -846,6 +969,21 @@ class PrettyTable:
         self._validate_option("horizontal_char", val)
         self._horizontal_char = val
 
+    @property
+    def horizontal_align_char(self):
+        """The character used to indicate column alignment in horizontal lines
+
+        Arguments:
+
+        horizontal_align_char - single character string used to indicate alignment"""
+        return self._bottom_left_junction_char or self.junction_char
+
+    @horizontal_align_char.setter
+    def horizontal_align_char(self, val):
+        val = str(val)
+        self._validate_option("horizontal_align_char", val)
+        self._horizontal_align_char = val
+
     @property
     def junction_char(self):
         """The character used when printing table borders to draw line junctions
@@ -858,9 +996,138 @@ class PrettyTable:
     @junction_char.setter
     def junction_char(self, val):
         val = str(val)
-        self._validate_option("vertical_char", val)
+        self._validate_option("junction_char", val)
         self._junction_char = val
 
+    @property
+    def top_junction_char(self):
+        """The character used when printing table borders to draw top line junctions
+
+        Arguments:
+
+        top_junction_char - single character string used to draw top line junctions"""
+        return self._top_junction_char or self.junction_char
+
+    @top_junction_char.setter
+    def top_junction_char(self, val):
+        val = str(val)
+        self._validate_option("top_junction_char", val)
+        self._top_junction_char = val
+
+    @property
+    def bottom_junction_char(self):
+        """The character used when printing table borders to draw bottom line junctions
+
+        Arguments:
+
+        bottom_junction_char -
+            single character string used to draw bottom line junctions"""
+        return self._bottom_junction_char or self.junction_char
+
+    @bottom_junction_char.setter
+    def bottom_junction_char(self, val):
+        val = str(val)
+        self._validate_option("bottom_junction_char", val)
+        self._bottom_junction_char = val
+
+    @property
+    def right_junction_char(self):
+        """The character used when printing table borders to draw right line junctions
+
+        Arguments:
+
+        right_junction_char -
+            single character string used to draw right line junctions"""
+        return self._right_junction_char or self.junction_char
+
+    @right_junction_char.setter
+    def right_junction_char(self, val):
+        val = str(val)
+        self._validate_option("right_junction_char", val)
+        self._right_junction_char = val
+
+    @property
+    def left_junction_char(self):
+        """The character used when printing table borders to draw left line junctions
+
+        Arguments:
+
+        left_junction_char - single character string used to draw left line junctions"""
+        return self._left_junction_char or self.junction_char
+
+    @left_junction_char.setter
+    def left_junction_char(self, val):
+        val = str(val)
+        self._validate_option("left_junction_char", val)
+        self._left_junction_char = val
+
+    @property
+    def top_right_junction_char(self):
+        """The character used when printing table borders to draw top-right line junctions
+
+        Arguments:
+
+        top_right_junction_char -
+            single character string used to draw top-right line junctions"""
+        return self._top_right_junction_char or self.junction_char
+
+    @top_right_junction_char.setter
+    def top_right_junction_char(self, val):
+        val = str(val)
+        self._validate_option("top_right_junction_char", val)
+        self._top_right_junction_char = val
+
+    @property
+    def top_left_junction_char(self):
+        """
+        The character used when printing table borders to draw top-left line junctions
+
+        Arguments:
+
+        top_left_junction_char -
+            single character string used to draw top-left line junctions"""
+        return self._top_left_junction_char or self.junction_char
+
+    @top_left_junction_char.setter
+    def top_left_junction_char(self, val):
+        val = str(val)
+        self._validate_option("top_left_junction_char", val)
+        self._top_left_junction_char = val
+
+    @property
+    def bottom_right_junction_char(self):
+        """The character used when printing table borders
+           to draw bottom-right line junctions
+
+        Arguments:
+
+        bottom_right_junction_char -
+            single character string used to draw bottom-right line junctions"""
+        return self._bottom_right_junction_char or self.junction_char
+
+    @bottom_right_junction_char.setter
+    def bottom_right_junction_char(self, val):
+        val = str(val)
+        self._validate_option("bottom_right_junction_char", val)
+        self._bottom_right_junction_char = val
+
+    @property
+    def bottom_left_junction_char(self):
+        """The character used when printing table borders
+           to draw bottom-left line junctions
+
+        Arguments:
+
+        bottom_left_junction_char -
+            single character string used to draw bottom-left line junctions"""
+        return self._bottom_left_junction_char or self.junction_char
+
+    @bottom_left_junction_char.setter
+    def bottom_left_junction_char(self, val):
+        val = str(val)
+        self._validate_option("bottom_left_junction_char", val)
+        self._bottom_left_junction_char = val
+
     @property
     def format(self):
         """Controls whether or not HTML tables are formatted to match styling options
@@ -907,7 +1174,7 @@ class PrettyTable:
 
     @property
     def oldsortslice(self):
-        """ oldsortslice - Slice rows before sorting in the "old style" """
+        """oldsortslice - Slice rows before sorting in the "old style" """
         return self._oldsortslice
 
     @oldsortslice.setter
@@ -927,7 +1194,7 @@ class PrettyTable:
                 self._validate_option(option, kwargs[option])
                 options[option] = kwargs[option]
             else:
-                options[option] = getattr(self, "_" + option)
+                options[option] = getattr(self, option)
         return options
 
     ##############################
@@ -946,10 +1213,12 @@ class PrettyTable:
             self._set_markdown_style()
         elif style == ORGMODE:
             self._set_orgmode_style()
+        elif style == DOUBLE_BORDER:
+            self._set_double_border_style()
         elif style == RANDOM:
             self._set_random_style()
         else:
-            raise Exception("Invalid pre-set style!")
+            raise Exception("Invalid pre-set style")
 
     def _set_orgmode_style(self):
         self._set_default_style()
@@ -964,6 +1233,7 @@ class PrettyTable:
         self.right_padding_width = 1
         self.vertical_char = "|"
         self.junction_char = "|"
+        self._horizontal_align_char = ":"
 
     def _set_default_style(self):
 
@@ -976,7 +1246,16 @@ class PrettyTable:
         self.right_padding_width = 1
         self.vertical_char = "|"
         self.horizontal_char = "-"
+        self._horizontal_align_char = None
         self.junction_char = "+"
+        self._top_junction_char = None
+        self._bottom_junction_char = None
+        self._right_junction_char = None
+        self._left_junction_char = None
+        self._top_right_junction_char = None
+        self._top_left_junction_char = None
+        self._bottom_right_junction_char = None
+        self._bottom_left_junction_char = None
 
     def _set_msword_style(self):
 
@@ -996,6 +1275,19 @@ class PrettyTable:
         self.left_padding_width = 0
         self.right_padding_width = 8
 
+    def _set_double_border_style(self):
+        self.horizontal_char = "═"
+        self.vertical_char = "║"
+        self.junction_char = "╬"
+        self.top_junction_char = "╦"
+        self.bottom_junction_char = "╩"
+        self.right_junction_char = "╣"
+        self.left_junction_char = "╠"
+        self.top_right_junction_char = "╗"
+        self.top_left_junction_char = "╔"
+        self.bottom_right_junction_char = "╝"
+        self.bottom_left_junction_char = "╚"
+
     def _set_random_style(self):
 
         # Just for fun!
@@ -1053,7 +1345,7 @@ class PrettyTable:
         if row_index > len(self._rows) - 1:
             raise Exception(
                 f"Can't delete row at index {row_index}, "
-                f"table only has {len(self._rows)} rows!"
+                f"table only has {len(self._rows)} rows"
             )
         del self._rows[row_index]
 
@@ -1084,9 +1376,19 @@ class PrettyTable:
         else:
             raise Exception(
                 f"Column length {len(column)} does not match number of rows "
-                f"{len(self._rows)}!"
+                f"{len(self._rows)}"
             )
 
+    def add_autoindex(self, fieldname="Index"):
+        """Add an auto-incrementing index column to the table.
+        Arguments:
+        fieldname - name of the field to contain the new column of data"""
+        self._field_names.insert(0, fieldname)
+        self._align[fieldname] = self.align
+        self._valign[fieldname] = self.valign
+        for i, row in enumerate(self._rows):
+            row.insert(0, i + 1)
+
     def del_column(self, fieldname):
 
         """Delete a column from the table
@@ -1135,10 +1437,12 @@ class PrettyTable:
 
     def _format_value(self, field, value):
         if isinstance(value, int) and field in self._int_format:
-            value = ("%%%sd" % self._int_format[field]) % value
+            return ("%%%sd" % self._int_format[field]) % value
         elif isinstance(value, float) and field in self._float_format:
-            value = ("%%%sf" % self._float_format[field]) % value
-        return str(value)
+            return ("%%%sf" % self._float_format[field]) % value
+
+        formatter = self._custom_format.get(field, (lambda f, v: str(v)))
+        return formatter(field, value)
 
     def _compute_table_width(self, options):
         table_width = 2 if options["vrules"] in (FRAME, ALL) else 0
@@ -1239,14 +1543,14 @@ class PrettyTable:
 
         return rows
 
-    def _format_row(self, row, options):
+    def _format_row(self, row):
         return [
             self._format_value(field, value)
             for (field, value) in zip(self._field_names, row)
         ]
 
-    def _format_rows(self, rows, options):
-        return [self._format_row(row, options) for row in rows]
+    def _format_rows(self, rows):
+        return [self._format_row(row) for row in rows]
 
     ##############################
     # PLAIN TEXT STRING METHODS  #
@@ -1270,13 +1574,29 @@ class PrettyTable:
             Allowed values: FRAME, ALL, NONE
         int_format - controls formatting of integer data
         float_format - controls formatting of floating point data
+        custom_format - controls formatting of any column using callable
         padding_width - number of spaces on either side of column data (only used if
             left and right paddings are None)
         left_padding_width - number of spaces on left hand side of column data
         right_padding_width - number of spaces on right hand side of column data
         vertical_char - single character string used to draw vertical lines
         horizontal_char - single character string used to draw horizontal lines
+        horizontal_align_char - single character string used to indicate alignment
         junction_char - single character string used to draw line junctions
+        junction_char - single character string used to draw line junctions
+        top_junction_char - single character string used to draw top line junctions
+        bottom_junction_char -
+            single character string used to draw bottom line junctions
+        right_junction_char - single character string used to draw right line junctions
+        left_junction_char - single character string used to draw left line junctions
+        top_right_junction_char -
+            single character string used to draw top-right line junctions
+        top_left_junction_char -
+            single character string used to draw top-left line junctions
+        bottom_right_junction_char -
+            single character string used to draw bottom-right line junctions
+        bottom_left_junction_char -
+            single character string used to draw bottom-left line junctions
         sortby - name of field to sort rows by
         sort_key - sorting key function, applied to data points before sorting
         reversesort - True or False to sort in descending or ascending order
@@ -1296,7 +1616,7 @@ class PrettyTable:
         rows = self._get_rows(options)
 
         # Turn all data in all rows into Unicode, formatted as desired
-        formatted_rows = self._format_rows(rows, options)
+        formatted_rows = self._format_rows(rows)
 
         # Compute column widths
         self._compute_widths(formatted_rows, options)
@@ -1311,15 +1631,27 @@ class PrettyTable:
         if options["header"]:
             lines.append(self._stringify_header(options))
         elif options["border"] and options["hrules"] in (ALL, FRAME):
-            lines.append(self._hrule)
+            lines.append(self._stringify_hrule(options, where="top_"))
+            if title and options["vrules"] in (ALL, FRAME):
+                lines[-1] = (
+                    self.left_junction_char + lines[-1][1:-1] + self.right_junction_char
+                )
 
         # Add rows
-        for row in formatted_rows:
-            lines.append(self._stringify_row(row, options))
+        for row in formatted_rows[:-1]:
+            lines.append(self._stringify_row(row, options, self._hrule))
+        if formatted_rows:
+            lines.append(
+                self._stringify_row(
+                    formatted_rows[-1],
+                    options,
+                    self._stringify_hrule(options, where="bottom_"),
+                )
+            )
 
         # Add bottom of border
         if options["border"] and options["hrules"] == FRAME:
-            lines.append(self._hrule)
+            lines.append(self._stringify_hrule(options, where="bottom_"))
 
         if "orgmode" in self.__dict__ and self.orgmode is True:
             tmp = list()
@@ -1329,30 +1661,40 @@ class PrettyTable:
 
         return "\n".join(lines)
 
-    def _stringify_hrule(self, options):
+    def _stringify_hrule(self, options, where=""):
 
         if not options["border"]:
             return ""
         lpad, rpad = self._get_padding_widths(options)
         if options["vrules"] in (ALL, FRAME):
-            bits = [options["junction_char"]]
+            bits = [options[where + "left_junction_char"]]
         else:
             bits = [options["horizontal_char"]]
         # For tables with no data or fieldnames
         if not self._field_names:
-            bits.append(options["junction_char"])
+            bits.append(options[where + "right_junction_char"])
             return "".join(bits)
         for field, width in zip(self._field_names, self._widths):
             if options["fields"] and field not in options["fields"]:
                 continue
-            bits.append((width + lpad + rpad) * options["horizontal_char"])
+
+            line = (width + lpad + rpad) * options["horizontal_char"]
+
+            # If necessary, add column alignment characters (e.g. ":" for Markdown)
+            if self._horizontal_align_char:
+                if self._align[field] in ("l", "c"):
+                    line = self._horizontal_align_char + line[1:]
+                if self._align[field] in ("c", "r"):
+                    line = line[:-1] + self._horizontal_align_char
+
+            bits.append(line)
             if options["vrules"] == ALL:
-                bits.append(options["junction_char"])
+                bits.append(options[where + "junction_char"])
             else:
                 bits.append(options["horizontal_char"])
-        if options["vrules"] == FRAME:
+        if options["vrules"] in (ALL, FRAME):
             bits.pop()
-            bits.append(options["junction_char"])
+            bits.append(options[where + "right_junction_char"])
         return "".join(bits)
 
     def _stringify_title(self, title, options):
@@ -1362,10 +1704,10 @@ class PrettyTable:
         if options["border"]:
             if options["vrules"] == ALL:
                 options["vrules"] = FRAME
-                lines.append(self._stringify_hrule(options))
+                lines.append(self._stringify_hrule(options, "top_"))
                 options["vrules"] = ALL
             elif options["vrules"] == FRAME:
-                lines.append(self._stringify_hrule(options))
+                lines.append(self._stringify_hrule(options, "top_"))
         bits = []
         endpoint = (
             options["vertical_char"] if options["vrules"] in (ALL, FRAME) else " "
@@ -1383,7 +1725,13 @@ class PrettyTable:
         lpad, rpad = self._get_padding_widths(options)
         if options["border"]:
             if options["hrules"] in (ALL, FRAME):
-                bits.append(self._hrule)
+                bits.append(self._stringify_hrule(options, "top_"))
+                if options["title"] and options["vrules"] in (ALL, FRAME):
+                    bits[-1] = (
+                        self.left_junction_char
+                        + bits[-1][1:-1]
+                        + self.right_junction_char
+                    )
                 bits.append("\n")
             if options["vrules"] in (ALL, FRAME):
                 bits.append(options["vertical_char"])
@@ -1428,7 +1776,7 @@ class PrettyTable:
             bits.append(self._hrule)
         return "".join(bits)
 
-    def _stringify_row(self, row, options):
+    def _stringify_row(self, row, options, hrule):
 
         for (index, field, value, width) in zip(
             range(0, len(row)), self._field_names, row, self._widths
@@ -1503,7 +1851,7 @@ class PrettyTable:
 
         if options["border"] and options["hrules"] == ALL:
             bits[row_height - 1].append("\n")
-            bits[row_height - 1].append(self._hrule)
+            bits[row_height - 1].append(hrule)
 
         for y in range(0, row_height):
             bits[y] = "".join(bits[y])
@@ -1601,6 +1949,7 @@ class PrettyTable:
             Allowed values: FRAME, ALL, NONE
         int_format - controls formatting of integer data
         float_format - controls formatting of floating point data
+        custom_format - controls formatting of any column using callable
         padding_width - number of spaces on either side of column data (only used if
             left and right paddings are None)
         left_padding_width - number of spaces on left hand side of column data
@@ -1611,7 +1960,7 @@ class PrettyTable:
             <table> tag
         format - Controls whether or not HTML tables are formatted to match
             styling options (True or False)
-        xhtml - print <br/> tags if True, <br> tags if false"""
+        xhtml - print <br/> tags if True, <br> tags if False"""
 
         options = self._get_options(kwargs)
 
@@ -1633,9 +1982,7 @@ class PrettyTable:
         open_tag = ["<table"]
         if options["attributes"]:
             for attr_name in options["attributes"]:
-                open_tag.append(
-                    ' {}="{}"'.format(attr_name, options["attributes"][attr_name])
-                )
+                open_tag.append(f' {attr_name}="{options["attributes"][attr_name]}"')
         open_tag.append(">")
         lines.append("".join(open_tag))
 
@@ -1665,7 +2012,7 @@ class PrettyTable:
         # Data
         lines.append("    <tbody>")
         rows = self._get_rows(options)
-        formatted_rows = self._format_rows(rows, options)
+        formatted_rows = self._format_rows(rows)
         for row in formatted_rows:
             lines.append("        <tr>")
             for field, datum in zip(self._field_names, row):
@@ -1707,9 +2054,7 @@ class PrettyTable:
                 open_tag.append(' frame="vsides" rules="cols"')
         if options["attributes"]:
             for attr_name in options["attributes"]:
-                open_tag.append(
-                    ' {}="{}"'.format(attr_name, options["attributes"][attr_name])
-                )
+                open_tag.append(f' {attr_name}="{options["attributes"][attr_name]}"')
         open_tag.append(">")
         lines.append("".join(open_tag))
 
@@ -1740,7 +2085,7 @@ class PrettyTable:
         # Data
         lines.append("    <tbody>")
         rows = self._get_rows(options)
-        formatted_rows = self._format_rows(rows, options)
+        formatted_rows = self._format_rows(rows)
         aligns = []
         valigns = []
         for field in self._field_names:
@@ -1773,6 +2118,124 @@ class PrettyTable:
 
         return "\n".join(lines)
 
+    ##############################
+    # LATEX STRING METHODS       #
+    ##############################
+
+    def get_latex_string(self, **kwargs):
+        """Return string representation of LaTex formatted version of table in current
+        state.
+
+        Arguments:
+
+        start - index of first data row to include in output
+        end - index of last data row to include in output PLUS ONE (list slice style)
+        fields - names of fields (columns) to include
+        header - print a header showing field names (True or False)
+        border - print a border around the table (True or False)
+        hrules - controls printing of horizontal rules after rows.
+            Allowed values: ALL, FRAME, HEADER, NONE
+        vrules - controls printing of vertical rules between columns.
+            Allowed values: FRAME, ALL, NONE
+        int_format - controls formatting of integer data
+        float_format - controls formatting of floating point data
+        sortby - name of field to sort rows by
+        sort_key - sorting key function, applied to data points before sorting
+        format - Controls whether or not HTML tables are formatted to match
+            styling options (True or False)
+        """
+        options = self._get_options(kwargs)
+
+        if options["format"]:
+            string = self._get_formatted_latex_string(options)
+        else:
+            string = self._get_simple_latex_string(options)
+        return string
+
+    def _get_simple_latex_string(self, options):
+        lines = []
+
+        wanted_fields = []
+        if options["fields"]:
+            wanted_fields = [
+                field for field in self._field_names if field in options["fields"]
+            ]
+        else:
+            wanted_fields = self._field_names
+
+        alignments = "".join([self._align[field] for field in wanted_fields])
+
+        begin_cmd = "\\begin{tabular}{%s}" % alignments
+        lines.append(begin_cmd)
+
+        # Headers
+        if options["header"]:
+            lines.append(" & ".join(wanted_fields) + " \\\\")
+
+        # Data
+        rows = self._get_rows(options)
+        formatted_rows = self._format_rows(rows)
+        for row in formatted_rows:
+            wanted_data = [
+                d for f, d in zip(self._field_names, row) if f in wanted_fields
+            ]
+            lines.append(" & ".join(wanted_data) + " \\\\")
+
+        lines.append("\\end{tabular}")
+
+        return "\r\n".join(lines)
+
+    def _get_formatted_latex_string(self, options):
+        lines = []
+
+        wanted_fields = []
+        if options["fields"]:
+            wanted_fields = [
+                field for field in self._field_names if field in options["fields"]
+            ]
+        else:
+            wanted_fields = self._field_names
+
+        wanted_alignments = [self._align[field] for field in wanted_fields]
+        if options["border"] and options["vrules"] == ALL:
+            alignment_str = "|".join(wanted_alignments)
+        else:
+            alignment_str = "".join(wanted_alignments)
+
+        if options["border"] and options["vrules"] in [ALL, FRAME]:
+            alignment_str = "|" + alignment_str + "|"
+
+        begin_cmd = "\\begin{tabular}{%s}" % alignment_str
+        lines.append(begin_cmd)
+
+        if options["border"] and options["hrules"] in [ALL, FRAME]:
+            lines.append("\\hline")
+
+        # Headers
+        if options["header"]:
+            lines.append(" & ".join(wanted_fields) + " \\\\")
+        if options["border"] and options["hrules"] in [ALL, HEADER]:
+            lines.append("\\hline")
+
+        # Data
+        rows = self._get_rows(options)
+        formatted_rows = self._format_rows(rows)
+        rows = self._get_rows(options)
+        for row in formatted_rows:
+            wanted_data = [
+                d for f, d in zip(self._field_names, row) if f in wanted_fields
+            ]
+            lines.append(" & ".join(wanted_data) + " \\\\")
+            if options["border"] and options["hrules"] == ALL:
+                lines.append("\\hline")
+
+        if options["border"] and options["hrules"] == FRAME:
+            lines.append("\\hline")
+
+        lines.append("\\end{tabular}")
+
+        return "\r\n".join(lines)
+
 
 ##############################
 # UNICODE WIDTH FUNCTION     #
@@ -1935,6 +2398,6 @@ def from_html_one(html_code, **kwargs):
         assert len(tables) == 1
     except AssertionError:
         raise Exception(
-            "More than one <table> in provided HTML code!  Use from_html instead."
+            "More than one <table> in provided HTML code. Use from_html instead."
         )
     return tables[0]
